@@ -4,7 +4,7 @@
 """
 ===============================================================================
 
-    Original code copyright (C) 2009-2021 Rudolf Cardinal (rudolf@pobox.com).
+    Original code copyright (C) 2009-2022 Rudolf Cardinal (rudolf@pobox.com).
 
     This file is part of cardinal_pythonlib.
 
@@ -74,6 +74,7 @@ from typing import Any, Dict
 # Simple information for R
 # =============================================================================
 
+
 def get_python_repr(x: Any) -> str:
     r"""
     A few notes:
@@ -163,9 +164,9 @@ def test_get_dict() -> Dict[str, Any]:
 
     """
     return {
-        'integers': [1, 2, 3, 4, 5],
-        'floats': [1.1, 2.1, 3.1, 4.1, 5.1],
-        'strings': ["one", "two", "three", "four", "five"],
+        "integers": [1, 2, 3, 4, 5],
+        "floats": [1.1, 2.1, 3.1, 4.1, 5.1],
+        "strings": ["one", "two", "three", "four", "five"],
     }
 
 
